#!/bin/bash

set -e

function show_help() {
cat <<HelpInfo

Provide maping of linux OS from iso image to flash storage device.

    -h show help
    -d specifies flash storage device like /dev/sdb
       (installation should be provided on device, not a partition)
    -i iso file
    -e specify eraselist file
    -f create flashtool directory
    -r set firmware revision file

Example:
#:install_script -d /dev/sdb -i ./minimal_linux_live.iso -e ./eraselist

HelpInfo
}

flash_storage=""
iso_file=""
eraselist_file="" 
flashtool=false
fwrevision=""
confirmation=""
OPTINDEX=1

while getopts "hd:i:e:fr:" opt; do
	case "$opt" in
        h)
	    show_help
	    exit 0
            ;;
        d)
            flash_storage=$OPTARG
	    ;;
        i)
            iso_file=$OPTARG
            BASEDIR=$(dirname "$iso_file")
            NBASE=$(echo "$BASEDIR" | sed 's/ /\\ /g')		
            ;;
        e)
            eraselist_file=$OPTARG
            if [ ! -e "$eraselist_file" ]; then
                echo "Eraselist file $eraselist_file do not exist."
                exit -1
            fi
            ;;
        f)
            flashtool=true
            ;;
        r)
            fwrevision=$OPTARG
            ;;
        '?')
    		show_help &>2
	    	exit 1
	esac
done

		
if [[ -z $flash_storage || -z $iso_file ]]; then
	echo 'Not all arguments specified corectly.'
	show_help
	exit 1	
fi

if [[ `find $flash_storage -type b 2> /dev/null` != "$flash_storage" ]]; then
	echo "Selected device $flash_storage do not exist."
	exit 1	
fi

if [[ `find "$iso_file" -type f 2> /dev/null` != "$iso_file" ]]; then
	echo "Selected iso file $iso_file do not exist."
	exit 1	
fi

echo "All data on disk $flash_storage will be lost. Continue? (yes/no)"

read confirmation

if [[ "$confirmation" != "y" && "$confirmation" != "yes" ]]; then
	echo "Exiting..."
	exit 0
fi

for mdisk in `ls $flash_storage[1-9]` ; do
    if [ -n "`mount | grep $mdisk`" ] ; then
        umount $mdisk
    fi
done

if [ -z `command -v fdisk $flash_storage 2> /dev/null` ]; then
	echo "Unable to find fdisk utility, please install it."
	exit 1	
fi

if [ ! -e "$BASEDIR/syslinux" ]; then
	echo "Unable to find syslinux in current directory."
	exit 1	
fi

if [ ! -e "$BASEDIR/mbr.bin" ]; then
	echo "Unable to find mbr.bin in current directory."
	exit 1	
fi


sudo fdisk $flash_storage <<FormatFlashDrive
o
n
p
1
4096

a
w
FormatFlashDrive

flash_storage_disk="$flash_storage"1

sudo mkfs.msdos $flash_storage_disk

if [ -d /mnt/minimal_linux_live_iso ] ; then
    sudo rm -Rf /mnt/minimal_linux_live_iso 
fi

if [ -d /mnt/minimal_linux_live_disk ] ; then
    sudo rm -Rf /mnt/minimal_linux_live_disk 
fi

sudo mkdir /mnt/minimal_linux_live_iso

sudo mkdir /mnt/minimal_linux_live_disk

sudo mount -o loop "$iso_file" /mnt/minimal_linux_live_iso
	
sudo mount "$flash_storage_disk" /mnt/minimal_linux_live_disk

if [ -e "$eraselist_file" ]; then
    sudo mkdir /mnt/minimal_linux_live_disk/eraselist
    sudo cp "$eraselist_file" /mnt/minimal_linux_live_disk/eraselist/
fi

if $flashtool ; then

    sudo mkdir /mnt/minimal_linux_live_disk/fwimages/
    
    if [ -e "$fwrevision" ]; then
        sudo cp "$fwrevision" /mnt/minimal_linux_live_disk/fwimages/revision
    fi
fi

sudo cp /mnt/minimal_linux_live_iso/linux /mnt/minimal_linux_live_disk/

sudo cp /mnt/minimal_linux_live_iso/initrd.img /mnt/minimal_linux_live_disk/ 

sudo cp /mnt/minimal_linux_live_iso/isolinux.cfg /mnt/minimal_linux_live_disk/syslinux.cfg

sudo umount /mnt/minimal_linux_live_iso/

sudo umount /mnt/minimal_linux_live_disk/

sudo rmdir /mnt/minimal_linux_live_iso

sudo rmdir /mnt/minimal_linux_live_disk
if [ -e /tmp/syslinux_tmp ] ; then
   rm /tmp/syslinux_tmp
fi

ln -s "$BASEDIR/syslinux" /tmp/syslinux_tmp
sudo /tmp/syslinux_tmp -m -a -i $flash_storage_disk 

rm /tmp/syslinux_tmp

sudo dd bs=440 count=1 conv=notrunc if="$BASEDIR/mbr.bin" of="$flash_storage"

printf "\n\n\n"

echo "To proceed with secure erase, power off your computer, and then boot from USB drive."

echo "Do you want to reboot now [yes/no]? "

read confirmation

if [[ "$confirmation" != "y" && "$confirmation" != "yes" ]]; then
    exit 0
fi

echo "Rebooting..."
sudo reboot

